#******************************************************************************
#   @file   EVAL_ADMP441Z_Demo.tcl
#   @brief  tcl script used for generating a wave file
#   @author ATofan (alexandru.tofan@analog.com)
#
#******************************************************************************
# Copyright 2011(c) Analog Devices, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#  - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#  - Neither the name of Analog Devices, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#  - The use of this software may or may not infringe the patent rights
#    of one or more patent holders.  This license does not release you
#    from the requirement that you obtain separate licenses from these
#    patent holders to use this software.
#  - Use of the software either in source or binary form, must be run
#    on or directly connected to an Analog Devices Inc. component.
#
# THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#******************************************************************************

puts "Running data capture script for EVAL-ADMP441Z..."

puts "Connecting to the processor..."
	connect mb mdm

puts "Downloading the MicroBlaze program..."
	rst
	stop
	rst
	stop
	dow PmodMIC2.elf
	
	# Setting startTest to test_ptr[]
	set startTest [mrd_var admp441DataPointer]
	set startTest [format "0x%08x" $startTest]

puts "Running program for 25 seconds..."	
	con
	after 25000
	stop

puts "Recorded data successfully stored in RAM"

puts "Generating EVAL_ADMP441Z_Demo.wav"	
puts "Reading data from RAM"	
puts "Moving data into file EVAL_ADMP441Z_Demo.wav "
		
puts -nonewline "Writing WAVE Header..."		
	# Write WAVE Header
	set data {}
	set micData [mrd $startTest 11]
	for {set i 1} {$i < 22} {incr i 2} {
		set val [lindex $micData $i]
		if { ![string length $val] } {
			set val "0"
		} 
		set val [expr 0x$val] 
		lappend data $val 
		lappend data "\n" 
		set addr [expr $startTest + 4]
		set startTest [format "0x%08x" $addr]
	}		 
	set fp [ open EVAL_ADMP441Z_Demo.wav a ]
	fconfigure $fp -translation binary
	for { set index 0 } { $index <= 21 } { incr index 2} {
		set byte_1 [ lindex $data $index ]
		set binary_first_16_bits [binary format S1 [expr {($byte_1 >> 16)}]]
		set binary_second_16_bits [binary format S1 [expr {(($byte_1 << 16) >> 16)}]]
		puts -nonewline $fp $binary_first_16_bits
		puts -nonewline $fp $binary_second_16_bits
	}
	close $fp

puts "Done!"

puts "Writing Audio Data"
puts "Operation will take a few minutes"
        
	 #Write audio data	
	 for {set j 0} {$j < 5000} {incr j 1} {		
		set data {}
        puts $j
		set micData [mrd $startTest 100]
		for {set i 1} {$i < 200} {incr i 2} {
			set val [lindex $micData $i]
			if { ![string length $val] } {
				set val "0"
			} 
			set val [expr 0x$val]  
			lappend data $val 
			lappend data "\n" 
			set addr [expr $startTest + 4]
			set startTest [format "0x%08x" $addr]
		}		 
		set fp [ open EVAL_ADMP441Z_Demo.wav a ]
		fconfigure $fp -translation binary
		for { set index 0 } { $index <= 199 } { incr index 2} {
			set byte_1 [ lindex $data $index ]
			set binary_first_16_bits [binary format S1 [expr {($byte_1 >> 16)}]]
			set binary_second_16_bits [binary format S1 [expr {(($byte_1 << 16) >> 16)}]]
			puts -nonewline $fp $binary_first_16_bits
			puts -nonewline $fp $binary_second_16_bits
		}
		close $fp
	  }	

puts "Data acquisition is complete. Data is stored in the file EVAL_ADMP441Z_Demo.wav."	

puts "Disconnecting from the processor..." 
	stop
	
puts ">>>"
puts "Press CTRL-C and type Y"
	after 20000