/***************************************************************************//**
*   @file   main.c
*   @brief  Main File for ADMP441 Driver.
*   @author ATofan (alexandru.tofan@analog.com)
********************************************************************************
* Copyright 2012(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
********************************************************************************
*   SVN Revision: $WCREV$
*******************************************************************************/

/*****************************************************************************/
/***************************** Include Files *********************************/
/*****************************************************************************/
#include <stdio.h>
#include "xparameters.h"
#include "xil_cache.h"
#include "xil_io.h"
#include "mb_interface.h"
#include "mb_interrupts.h"
#include "admp441.h"

volatile int rxData;

unsigned int nrSamples = 488000;
unsigned int nrSeconds = 30;

volatile int admp441Data[1000044] = {0,};
volatile int* admp441DataPointer = &admp441Data[0];
volatile int admp441RxData;
volatile int admp441DataLitte;
volatile char admp441RxEn = 0;
volatile int ampFactor = 5;

volatile unsigned int i = 0;

/******************************************************************************
* @brief Main function.
*
* @return Always returns 0.
******************************************************************************/
int main()
{

	Xil_ICacheEnable();
	Xil_DCacheEnable();

	xil_printf("PmodMIC2 Demo Program\n\r");


	// Enable Interrupts
	enableInterrupts();
	// Enable microphone
	Xil_Out32(XPAR_ADMP441_0_BASEADDR, 0x01);
	// Prepare WAV Header
	xil_printf("Preparing WAV File Header...");
	admp441Data[0]  = CHUNK_RIFF_ID;
	admp441Data[1]  = CHUNK_SIZE;
	admp441Data[2]  = WAVE_FORMAT;
	admp441Data[3]  = SUBCHUNK1_ID;
	admp441Data[4]  = SUBCHUNK1_SIZE;
	admp441Data[5]  = (AUDIO_FORMAT << 16) | (NR_CHANNELS);
	admp441Data[6]  = SAMPLE_RATE;
	admp441Data[7]  = BYTE_RATE;
	admp441Data[8]  = (BLOCK_ALIGN << 16) | (NR_BITS);
	admp441Data[9]  = SUBCHUNK2_ID;
	admp441Data[10] = SUBCHUNK2_SIZE;
	xil_printf(" Done!\n\r");
	// Set number of samples
	nrSamples = nrSeconds * 48800;
	// Start from admp441Data[11]
	i = 11;
	xil_printf("Receiving Data... ");
	// Enable data reception
	admp441RxEn = 1;
	// Acquire data and save in RAM
	while(i < nrSamples + 11)
	{
		admp441DataLitte = bigToLittle(admp441RxData);
		admp441Data[i] = admp441DataLitte;
	}
	// Disable data reception
	admp441RxEn = 0;

	xil_printf("Done!\n\r");
	xil_printf("Samples Acquired: %d\n\r", nrSamples);

	while(1)
	{

	}

	/*for(i=0; i<10; i++)
	{
		Xil_Out32(XPAR_D_SHARED_MEM_BUS_0_PSRAM_BASEADDR+i*4, 0xA5F8);
	}

	for(i=0; i<10; i++)
	{
		xil_printf("%x\n\r",Xil_In32(XPAR_D_SHARED_MEM_BUS_0_PSRAM_BASEADDR+i*4));
	}*/

	Xil_DCacheDisable();
	Xil_ICacheDisable();

	return 0;
}

